%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Name: BMSPoisInv.m		Created 99 - 02 - 04
%
%  Usage:  Implements one-dimensional version of Bayesian 
%          Multiscale algorithm for solution of Poisson
%	   linear inverse problems, as described in
%	     Nowak, R.D. and Kolaczyk, E.D. (2000) "A Statistical
% 	     Multiscale Framework for Poisson Inverse Problems,"
%	     IEEE Transactions on Information Theory, 46:5, 1811-1825.
%
%  Inputs:  y = vector of observed data (length n, n > 0 integer)
%      	    b = background intensity (length n)
%               Note: If no background information is to be included
%                     in the model, take b to be a zero vector.
%     	    P = matrix (nxm) of transition probabilities
%               Note: (i)   row=output, col=input
%	 	      (ii)  m = 2^J, J>0
%                     (iii)  column sums should be normalized to equal 1.0
%		      (iv)  zero-rows should be removed.
%	    q = vector of shape parameters for non-pointmass, symmetric
%	         beta prior (length J).
%           g = vector of mixture probabilities (length J), in the case
%               that a mixture of a point-mass and a symmetric beta
%               are to be used for prior distributions.
%               Note: To use just a single symmetric beta distribution as a
%                     prior, set g = 0.99999*ones(1,J);
%  Outputs:   est = MAP estimate of underlying emission intensity
%  Calls:  Requires the Matlab scripts BMS_EMStep.m and circ_conv.m
%
%  Comments: This code implements the estimation method of the 
%            above referenced paper in a straightforward fashion.
%            In particular, no attempt has been made to optimize the
%            code beyond fairly standard measures, nor is the code
%            equipped with the various flags, bells, and whistles of a
%            more professional quality implementation.  Users will likely
%            wish to insert command lines to produce output for tracking
%            progress of the underlying EM algorithm.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function est = BMSPoisInv(y, b, P, q, g)

% Initialize parameters.
sz = size(P);
nlam = mean(y)*ones(sz(2),1);
err=[]; 
tol = 1.0e-6;
maxit = 1000;

% Iterate EM algorithm.
iter=0;
convcrit = 100;
while ( (convcrit > tol) & (iter < maxit) ),
        lam  = nlam;
	nlam = BMS_EMStep(lam,b,P,y,q,g);
	err(iter+1) = norm(nlam-lam).^2 ./ norm(lam).^2;
        convcrit = err(iter+1);
        iter=iter+1;
end

disp(['Number of iterations until convergence: ' num2str(iter)] )

disp(['Relative Error at Convergence: ' num2str(convcrit)] )

est = nlam;

%  This code has been developed by Robert Nowak and Eric Kolaczyk.
%  Copyright (c) Robert Nowak and Eric Kolaczyk, 1999.
%  All rights reserved.

