%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Name: BMS_EMStep.m		Created 99 - 02 - 04.
%  
%  Usage:  Performs EM update for iterative calculation of
%          Bayesian multiscale MAP estimate in Nowak/Kolaczyk
%          algorithm.  Called by BMSPoisInv.m
%
%  Input:  lam = current estimate of intensity
%	   b = background intensity (in space of observed data)
%	   P = matrix of transition probabilities
%	   y = observed data (counts) 
%	   q = vector of shape parameters for non-pointmass, symmetric
%              beta prior
%	   g = vector of mixture probabilities, in the case that a
%              mixture of pointmass and symmetric beta distributions
%              are to be used as priors.
%  Output:  nlam = new estimate of intensity
%	
%  Calls: Requires Matlab script circ_conv.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function nlam = BMS_EMStep(lam,b,P,y,q,g)

%% E-Step: Calculate complete data.
plam = P*lam + b;
z = (y./plam)*lam'.*P;

% Calculate hidden counts.
hz = sum(z);

% M-Step: Iteratively, by scale j, build up new estimate of lambda.
J = log2(length(lam));
R(1) = sum(sum(z));
for j=1:J-1
  lbb = betaln(q(j),q(j));

  zz = circ_conv(hz,[zeros(1,2^(J-j)) ones(1,2^(J-j))]);
  z1 = zz(1:2^(J-j+1):2^J);
  z2 = zz(2^(J-j)+1:2^(J-j+1):2^J);
	
  l1 = (z1+z2)*log(1/2) + ...
       log(  exp( log(g(j)) - lbb + (2*(q(j)-1))*log(1/2) ) + ...
       (1-g(j)) + eps );

  eR = (z1+q(j)-1)./(z1+z2+2*(q(j)-1)+eps);

  l2  = z1.*log(eR+eps)+z2.*log(1-eR+eps)+ ...
        log(g(j)) - lbb +(q(j)-1)*log(eR+eps)+(q(j)-1)*log(1-eR+eps);

  R(2^(j-1)+1:2^j) = 1/2*(l1 > l2)+(l1 <= l2).*eR;
end

% Final scale j=J.
z1 = hz(1:2:2^J);
z2 = hz(2:2:2^J);

lbb = betaln(q(J),q(J));

l1  = (z1+z2)*log(1/2)+ ...
      log(  exp( log(g(J)) - lbb + (2*(q(J)-1))*log(1/2) ) + ...
            (1-g(J))+eps);

eR = (z1+q(J)-1)./(z1+z2+2*(q(J)-1)+eps);
	
l2 = z1.*log(eR+eps)+z2.*log(1-eR+eps)+log(g(J)) - lbb + ...
     (q(J)-1)*log(eR+eps)+(q(J)-1)*log(1-eR+eps);

R(2^(J-1)+1:2^J) = (l1 > l2)*1/2 + (l1 <= l2).*eR;


nlam = R(1);
ulam = nlam;
for j=1:J
	ulam(1:2:2^j) = nlam.*R(2^(j-1)+1:2^(j));
	ulam(2:2:2^j) = nlam.*(1-R(2^(j-1)+1:2^(j)));

	nlam = ulam;
end
nlam = nlam';

%  This code has been developed by Robert Nowak and Eric Kolaczyk.
%  Copyright (c) Robert Nowak and Eric Kolaczyk, 1999.
%  All rights reserved.

